///File : MainFrame.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainFrame.hpp"
//------------------------------------------------------------------------------
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size, long style) : wxFrame(NULL, -1, title, pos, size, style)
{
    // Copie des fichiers dans ~/.devimaker... Si il faut
    if(findConfigFile(_T("sectionDB.xml"), false) == _T("")) // Recherche autre que data dir
    {
        copyToUserConfigDir(findFile(_T("data/sectionDB.xml")));
    }

    if(findConfigFile(_T("itemDB.xml"), false) == _T("")) // Recherche autre que data dir
    {
    	copyToUserConfigDir(findFile(_T("data/itemDB.xml")));
    }

    // Devis manager
        // Trouver le chemin des .xml pr le devis
    std::string sectionsDbPath = filenameWxToStd(findConfigFile(_T("sectionDB.xml")));
    std::string itemsDbPath = filenameWxToStd(findConfigFile(_T("itemDB.xml")));

        // Chargement
    m_IRDBManager = new ItemSectionDBManager(sectionsDbPath, itemsDbPath);

        // Test
    if(!m_IRDBManager->IsOk())
    {
        wxMessageBox(_("Error : can't load estimate's data"), _("Error..."),  wxOK | wxICON_ERROR);
        exit(1);
    }

    // Presentation général
    wxString ImgFileName;
    #if defined(__WXMSW__)
        SetIcon(wxIcon(_T("monicone")));
    #elif defined(__UNIX__)
        ImgFileName = findFile(_T("data/devisDBedit-icon.png"));
        if(wxFileExists(ImgFileName))
        {
            SetIcon(wxIcon(ImgFileName, wxBITMAP_TYPE_PNG));
        }
    #endif //#ifdef WIN32
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));

    // Menus
    wxMenu *FileMenu = new wxMenu;
    FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_save.png"), wxID_SAVE)); // FileMenu->Append(wxID_SAVE);
    FileMenu->AppendSeparator();
    FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_quit.png"), wxID_EXIT)); // FileMenu->Append(wxID_EXIT);

    wxMenu *EditMenu = new wxMenu;
    m_undoStr = wxGetTranslation(_T("&Undo"));
    m_undoKey = _T("\tCtrl+Z");
    m_ItemUndo = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_undo.png"), wxID_UNDO, m_undoStr + m_undoKey));  // EditMenu->Append(wxID_UNDO, m_undoStr + m_undoKey);
    m_redoStr = wxGetTranslation(_T("&Redo"));
    m_redoKey = _T("\tCtrl+Y");
    m_ItemRedo = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_redo.png"), wxID_REDO, m_redoStr + m_redoKey));  // EditMenu->Append(wxID_REDO, m_redoStr + m_redoKey);
    EditMenu->AppendSeparator();
    m_ItemSelectAll = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_select_all.png"), wxID_SELECTALL, wxGetTranslation(_T("Select all")) + wxString(_T("\tCtrl+A"))));// EditMenu->Append(wxID_SELECTALL, wxGetTranslation(_T("Select all")) + wxString(_T("\tCtrl+A")));

        // Help Menu
    wxMenu *HelpMenu = new wxMenu;
    HelpMenu->Append(CreateImageMenuItem(HelpMenu, _T("menu_help.png"), wxID_HELP)); // HelpMenu->Append(wxID_HELP);
    HelpMenu->AppendSeparator();
    HelpMenu->Append(CreateImageMenuItem(HelpMenu, _T("menu_about.png"), wxID_ABOUT)); // HelpMenu->Append(wxID_ABOUT);

    wxMenuBar *MenuBar = new wxMenuBar();
    MenuBar->Append(FileMenu,_("&File"));
    MenuBar->Append(EditMenu,_("&Edit"));
    MenuBar->Append(HelpMenu,_("&Help"));

    SetMenuBar(MenuBar);

    // Corp
	wxBoxSizer* MainSizer = new wxBoxSizer( wxVERTICAL );

    m_MainNotebook = new wxNotebook(this, wxID_ANY);
    MainSizer->Add(m_MainNotebook, 1, wxEXPAND);

    m_PanelItems = new MainPanel(m_MainNotebook, wxID_ANY, m_IRDBManager, true);
    m_MainNotebook->AddPage(m_PanelItems, _("Items"), true);

    m_PanelSections = new MainPanel(m_MainNotebook, wxID_ANY, m_IRDBManager, false);
    m_MainNotebook->AddPage(m_PanelSections, _("Sections"));

    SetSizer( MainSizer );

    Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler(MainFrame::OnFrameClose));

    Connect( wxID_EXIT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnQuit));
    Connect( wxID_ABOUT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnAbout));
    Connect( wxID_SAVE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnSave));
    Connect( wxID_UNDO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnUndo));
    Connect( wxID_REDO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnRedo));
    Connect( wxID_HELP, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnHelp));
    Connect( wxID_SELECTALL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnSelectAll));

    // Pas de m_MainNotebook->Connect si non, segmentation fault
    Connect( m_MainNotebook->GetId(), wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED, wxCommandEventHandler(MainFrame::OnNotebookChange));
}
//------------------------------------------------------------------------------
void MainFrame::OnUndo(wxCommandEvent& event)
{
    if( m_IRDBManager->BackupCanGoTo(-1))
    {
        m_IRDBManager->BackupLoad(-1);
    }

    m_PanelItems->FeedTbl();
    m_PanelSections->FeedTbl();
}
//------------------------------------------------------------------------------
void MainFrame::OnRedo(wxCommandEvent& event)
{
    if( m_IRDBManager->BackupCanGoTo(1))
    {
        m_IRDBManager->BackupLoad(1);
    }

    m_PanelItems->FeedTbl();
    m_PanelSections->FeedTbl();
}
//------------------------------------------------------------------------------
void MainFrame::OnHelp(wxCommandEvent& event)
{
    wxString FileName = findFile(_T("help/fr/devismaker.hhp"));
    if(wxFileExists(FileName))
    {
        wxHtmlHelpController *myHelp = new wxHtmlHelpController(wxHF_DEFAULT_STYLE, this);
        myHelp->AddBook(FileName);
        myHelp->Display(_T("index.html"));
    }
}
//------------------------------------------------------------------------------
wxMenuItem* MainFrame::CreateImageMenuItem( wxMenu *parentMenu, wxString dataFilename, const int &id, const wxString &value) const
{
    wxMenuItem *item = new wxMenuItem(parentMenu, id, value);
    wxString ImgFileName = findFile(_T("data/menu/") + dataFilename);
    if(wxFileExists(ImgFileName))
    {
        wxImage img(ImgFileName, wxBITMAP_TYPE_PNG);
        item->SetBitmap(wxBitmap(img));
    }

    return item;
}
//------------------------------------------------------------------------------
void MainFrame::OnSelectAll(wxCommandEvent& event)
{
    m_PanelItems->SelectAll();
}
//------------------------------------------------------------------------------
wxString MainFrame::GetBackupDescription(const int &pos)
{
    wxString description;
    switch( m_IRDBManager->BackupGetType(pos))
    {
        case BACKUP_ITEM_DB_MODIFIED:
            description = wxString::Format(_("Item %s modified"), StrStdToWx(m_IRDBManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_DB_MODIFIED:
            description = wxString::Format(_("Section %s modified"), StrStdToWx(m_IRDBManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_ITEM_DB_ADDED:
            description = wxString::Format(_("Item %s added"), StrStdToWx(m_IRDBManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_DB_ADDED:
            description = wxString::Format(_("Section %s added"), StrStdToWx(m_IRDBManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_ITEM_DB_REMOVED:
            description = wxString::Format(_("Item %s removed"), StrStdToWx(m_IRDBManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_DB_REMOVED:
            description = wxString::Format(_("Section %s removed"), StrStdToWx(m_IRDBManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_NEW:
            description = _("Start");
            break;
    }

    return wxString(_T(" (")) + description + wxString(_T(")"));
}
//------------------------------------------------------------------------------
void MainFrame::MyRefreshMenu()
{
    /*#if defined(__WXMSW__) // Bug avec msw : les icones des menus ne sont pas gardé en cas de changement de label
    wxBitmap undoBmp = m_ItemUndo->GetBitmap();
    wxBitmap redoBmp = m_ItemRedo->GetBitmap();
    #endif*/

    if(m_IRDBManager->BackupCanGoTo(-1))
    {
        m_ItemUndo->SetItemLabel(m_undoStr + GetBackupDescription(-1) + m_undoKey);
    }
    else
    {
        m_ItemUndo->SetItemLabel(m_undoStr + m_undoKey);
    }

    if(m_IRDBManager->BackupCanGoTo(1))
    {
        m_ItemRedo->SetItemLabel(m_redoStr + GetBackupDescription(1) + m_redoKey);
    }
    else
    {
        m_ItemRedo->SetItemLabel(m_redoStr + m_redoKey);
    }

    /*#if defined(__WXMSW__) // Bug avec msw : les icones des menus ne sont pas gardé en cas de changement de label
    m_ItemUndo->SetBitmap(undoBmp);
    m_ItemRedo->SetBitmap(redoBmp);
    #endif*/

    m_ItemRedo->Enable(m_IRDBManager->BackupCanGoTo(1));
    m_ItemUndo->Enable(m_IRDBManager->BackupCanGoTo(-1));
    if(m_MainNotebook->GetPageCount() > 0) // La fn peut être appeleée par les MainPanel alors que la notebook est vide
    {
        m_ItemSelectAll->Enable(m_MainNotebook->GetCurrentPage() == m_PanelItems);
    }
}
//------------------------------------------------------------------------------
MainFrame::~MainFrame()
{
    delete m_IRDBManager;
}
//------------------------------------------------------------------------------
bool MainFrame::AskToSave()
{
    if(m_IRDBManager->IsOk() && m_IRDBManager->IsModified())
    {
        long rep = wxMessageBox(_("Do you want to save your work ?"), _("Close"), wxICON_WARNING | wxYES_NO | wxCANCEL | wxYES_DEFAULT, this);
        if(rep == wxCANCEL)
        {
            return false; // Rien
        }
        else if(rep == wxYES)
        {
            wxCommandEvent tmpEvt;
            OnSave(tmpEvt);
        }
    }
    return true; // On peut continuer
}
//------------------------------------------------------------------------------
void MainFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    if(AskToSave())
    {
        exit (EXIT_SUCCESS);
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnNotebookChange(wxCommandEvent& event)
{
    MyRefreshMenu();
}
//------------------------------------------------------------------------------
void MainFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    DialogAbout MyDialogAbout(this, wxID_ANY);
    MyDialogAbout.ShowModal();
}

//------------------------------------------------------------------------------
void MainFrame::OnSave(wxCommandEvent& event)
{
    m_IRDBManager->SaveDBs();
}
//------------------------------------------------------------------------------

void MainFrame::OnFrameClose(wxCloseEvent& event)
{
    if(AskToSave())
    {
        event.Skip();
    }
    else
    {
        event.Veto();
    }
}

