///File : MainFrame.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainFrame.hpp"
//------------------------------------------------------------------------------
void MainFrame::EnableMenus(bool enable)
{
    m_MenuItemSave->Enable(enable);
    m_MenuItemSaveAs->Enable(enable);
    m_MenuItemClose->Enable(enable);
    m_MenuItemPrintPreview->Enable(enable);
    m_MenuItemPrint->Enable(enable);
    m_MenuItemDevisProperties->Enable(enable);
    m_MenuItemDevisPrices->Enable(enable);
    m_MenuItemPrintSetup->Enable(enable);
    #if defined USE_MULTIPLE_FILE
    m_MenuItemSaveAll->Enable(enable);
    m_MenuItemCloseAll->Enable(enable);
    #endif
}
//------------------------------------------------------------------------------
#if defined USE_MULTIPLE_CONFIG
void MainFrame::FeedCfgListMenu()
{
    // On vide
    while( ! m_MenuItemsCfgs.empty())
    {
        m_cfgListMenu->Destroy(m_MenuItemsCfgs[m_MenuItemsCfgs.size() - 1]);
        m_MenuItemsCfgs.pop_back();
    }

    // On remplie
    std::vector<std::string> stdList = m_DevisManager->GetGeneralCfg()->GetCfgTbl();
    int currentCfgPos = m_DevisManager->GetGeneralCfg()->GetCurrentCfgPos();
    m_MenuItemsCfgs.resize(stdList.size());

    for(int i = 0; i < stdList.size(); i++)
    {
        m_MenuItemsCfgs[i] = m_cfgListMenu->AppendRadioItem(wxID_ANY, StrStdToWx(stdList[i]));
        Connect( m_MenuItemsCfgs[i]->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnMenuItemsSelectCfg));

        if(i == currentCfgPos)
        {
            m_MenuItemsCfgs[i]->Check(true);
        }
    }
}
#endif
//------------------------------------------------------------------------------
void MainFrame::MyRefresh(const bool &withChild)
{
    EnableMenus(m_DevisManager->IsOpen());
    wxString title = PROG_FULL_NAME_GETTEXT;

    #if (IS_DEBUG==1)
       title += _T(" - Debug");
    #endif

    title << _T(" - ") << BUILD_VERSION_NAME;

    if(m_DevisManager->IsOpen())
    {
        wxString name = StrStdToWx(m_DevisManager->GetDevisCfg()->GetString("name"));

        if(name != _T(""))
        {
            title << _T(" - ") << StrStdToWx(m_DevisManager->GetDevisCfg()->GetString("name"));
        }
        else
        {
            title << _T(" - ") << (_("Unnamed"));
        }
    }

    SetTitle(title);

    RefreshUndoRedo();

    #if defined USE_MULTIPLE_CONFIG
    FeedCfgListMenu();
    #endif

    #if defined USE_MULTIPLE_FILE
    RefreshNotebook();
    #endif

    #if defined(__WXMSW__)
    m_MenuBar->Refresh();
    #endif

    if(withChild && m_DevisManager->IsOpen())
    {
        m_MainPanel->MyRefresh();
    }
}

//------------------------------------------------------------------------------
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size, long style) : wxFrame(NULL, -1, title, pos, size, style)
{
    // Printing
    //m_printData.SetPaperId(wxPAPER_A4); // Tjr en A4 pas defaut.
    m_printDialogData.SetPrintData(m_printData);
    m_pageSetupDialogData.SetPrintData(m_printData);

    // Devis manager
        // Trouver le chemin des .xml pr le devis
    std::string sectionsDbPath = filenameWxToStd(findConfigFile(_T("sectionDB.xml")));
    std::string itemsDbPath = filenameWxToStd(findConfigFile(_T("itemDB.xml")));
    std::string generalCfgPath = filenameWxToStd(findConfigFile(_T("generalCfg.xml")));

        // Chargement
    m_DevisManager = new DevisManager(sectionsDbPath, itemsDbPath, generalCfgPath);

        // Test
    if(!m_DevisManager->IsOk())
    {
        wxMessageBox(_("Error : can't load Invoice's / Estimate's data !"), _("Error..."),  wxOK | wxICON_ERROR, this);
        Close();
    }



    Maximize(true);

    // Presentation général
    wxString ImgFileName;
    #if defined(__WXMSW__)
        SetIcon(wxIcon(_T("monicone")));
    #elif defined(__UNIX__)
        ImgFileName = findFile(_T("data/devismaker-icon.png"));
        if(wxFileExists(ImgFileName))
        {
            SetIcon(wxIcon(ImgFileName, wxBITMAP_TYPE_PNG));
        }
    #endif //#ifdef WIN32
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));

        // File menu
    wxMenu *FileMenu = new wxMenu;
    FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_new.png"), wxID_NEW));  // FileMenu->Append(wxID_NEW);
    FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_open.png"), wxID_OPEN)); // FileMenu->Append(wxID_OPEN);
    m_MenuItemSave = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_save.png"), wxID_SAVE)); // FileMenu->Append(wxID_SAVE);
    m_MenuItemSaveAs = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_save_as.png"), wxID_SAVEAS)); // FileMenu->Append(wxID_SAVEAS);
    #if defined USE_MULTIPLE_CONFIG
    m_MenuItemSaveAll = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_save_all.png"), wxID_ANY, _("Save all")));
    #endif
    FileMenu->AppendSeparator();
    m_MenuItemPrint = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_print.png"), wxID_PRINT)); // FileMenu->Append(wxID_PRINT);
    m_MenuItemPrintSetup = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_page_setup.png"), wxID_ANY, _("Page Set&up...")));
    m_MenuItemPrintPreview = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_print_preview.png"), wxID_PREVIEW)); // FileMenu->Append(wxID_PREVIEW);
    FileMenu->AppendSeparator();
    m_MenuItemClose = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_close.png"), wxID_CLOSE)); //FileMenu->Append(wxID_CLOSE);
    #if defined USE_MULTIPLE_CONFIG
    m_MenuItemCloseAll = FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_close_all.png"), wxID_ANY, _("Close all")));
    #endif
    FileMenu->AppendSeparator();
    FileMenu->Append(CreateImageMenuItem(FileMenu, _T("menu_quit.png"), wxID_EXIT)); // FileMenu->Append(wxID_EXIT);

        // Edit menu
    wxMenu *EditMenu = new wxMenu;
    m_undoStr = wxGetTranslation(_T("&Undo"));
    m_undoKey = _T("\tCtrl+Z");
    m_ItemUndo = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_undo.png"), wxID_UNDO, m_undoStr + m_undoKey));  // EditMenu->Append(wxID_UNDO, m_undoStr + m_undoKey);
    m_redoStr = wxGetTranslation(_T("&Redo"));
    m_redoKey = _T("\tCtrl+Y");
    m_ItemRedo = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_redo.png"), wxID_REDO, m_redoStr + m_redoKey));  // EditMenu->Append(wxID_REDO, m_redoStr + m_redoKey);
    EditMenu->AppendSeparator();
    m_MenuItemDevisProperties = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_properties.png"), wxID_PROPERTIES)); // EditMenu->Append(wxID_PROPERTIES );
    m_MenuItemDevisPrices = EditMenu->Append(CreateImageMenuItem(EditMenu, _T("menu_prices.png"), wxID_ANY, _("Invoice / Estimate pr&ices\tCtrl+I")));

        // Tools menu
    wxMenu *ToolsMenu = new wxMenu;
    ToolsMenu->Append(CreateImageMenuItem(ToolsMenu, _T("menu_refresh_db.png"), wxID_REFRESH, _("Refresh section and item &Data Base")));

    wxMenuItem *menuItemLunchDbEdit = CreateImageMenuItem(ToolsMenu, _T("menu_lunch_devisDBEdit.png"), wxID_ANY, _("Edit section and item &Data Base"));
    ToolsMenu->Append(menuItemLunchDbEdit);

    ToolsMenu->AppendSeparator();
    ToolsMenu->Append(CreateImageMenuItem(ToolsMenu, _T("menu_preferences.png"), wxID_PREFERENCES)); // ToolsMenu->Append(wxID_PREFERENCES);

    #if defined USE_MULTIPLE_CONFIG
    m_cfgListMenu = new wxMenu;

    wxMenuItem *MenuItemEditCfg = m_cfgListMenu->Append(CreateImageMenuItem(m_cfgListMenu, _T("menu_preferences.png"), wxID_EDIT)); // m_cfgListMenu->Append(wxID_EDIT);
    Connect( MenuItemEditCfg->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnDialogMultiCfgManager));

    m_cfgListMenu->AppendSeparator();

    wxMenuItem *menuItemMultiCfg = CreateImageMenuItem(ToolsMenu, _T("menu_multiCfg.png"), wxID_ANY, _("Configurations"));
    menuItemMultiCfg->SetSubMenu(m_cfgListMenu);
    ToolsMenu->Append(menuItemMultiCfg);
    #endif

        // Help Menu
    wxMenu *HelpMenu = new wxMenu;
    HelpMenu->Append(CreateImageMenuItem(HelpMenu, _T("menu_help.png"), wxID_HELP)); // HelpMenu->Append(wxID_HELP);
    HelpMenu->AppendSeparator();
    HelpMenu->Append(CreateImageMenuItem(HelpMenu, _T("menu_about.png"), wxID_ABOUT)); // HelpMenu->Append(wxID_ABOUT);

        // Menubar
    m_MenuBar = new wxMenuBar();
    m_MenuBar->Append(FileMenu,_("&File"));
    m_MenuBar->Append(EditMenu,_("&Edit"));
    m_MenuBar->Append(ToolsMenu,_("&Tools"));
    m_MenuBar->Append(HelpMenu,_("&Help"));

    SetMenuBar(m_MenuBar);

    // Body
	wxBoxSizer* MainSizer = new wxBoxSizer( wxVERTICAL );

    #if defined USE_MULTIPLE_FILE
    m_book = new wxAuiNotebook(this);

    m_book->Connect( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(MainFrame::OnNotebookChanged), NULL, this);
    m_book->Connect( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE, wxAuiNotebookEventHandler(MainFrame::OnNotebookClosing), NULL, this);
    m_book->Connect( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSED, wxAuiNotebookEventHandler(MainFrame::OnNotebookClosed), NULL, this);

    MainSizer->Add(m_book, 1, wxEXPAND);

    ImgFileName = findFile(_T("data/saved-16.png"));
    if(wxFileExists(ImgFileName))
    {
        m_bitmapSaved.LoadFile(ImgFileName, wxBITMAP_TYPE_PNG);
    }

    ImgFileName = findFile(_T("data/toSave-16.png"));
    if(wxFileExists(ImgFileName))
    {
        m_bitmapToSave.LoadFile(ImgFileName, wxBITMAP_TYPE_PNG);
    }
    #else
    m_MainPanel = new MainPanel(this, m_DevisManager);
    MainSizer->Add(m_MainPanel, 1, wxEXPAND);
	#endif

	SetSizer( MainSizer );

	// Drop files
    SetDropTarget(new MyFileDropTarget(this));

	// Events
	Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler(MainFrame::OnFrameClose));

    Connect( wxID_EXIT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnQuit));
    Connect( wxID_NEW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnNew));
    Connect( wxID_HELP, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnHelp));
    Connect( wxID_ABOUT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnAbout));
    Connect( wxID_OPEN, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnOpen));
    Connect( wxID_SAVE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnSave));
    Connect( wxID_SAVEAS, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnSaveAs));
    Connect( wxID_CLOSE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnClose));
    Connect( wxID_PREVIEW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnPrintPreview));
    Connect( wxID_PRINT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnPrint));
    Connect( wxID_PROPERTIES, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnDevisProperties));
    Connect( m_MenuItemDevisPrices->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnDevisPrices));
    Connect( menuItemLunchDbEdit->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnLunchDBEdit));
    Connect( m_MenuItemPrintSetup->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnPageSetup));
    Connect( wxID_REFRESH, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnRefresh));
    Connect( wxID_PREFERENCES, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnPreferences));

    Connect( wxID_UNDO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnUndo));
    Connect( wxID_REDO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnRedo));

    #if defined USE_MULTIPLE_CONFIG
    Connect( m_MenuItemSaveAll->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnSaveAll));
    Connect( m_MenuItemCloseAll->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainFrame::OnCloseAll));
    #endif

    MyRefresh();
}
//------------------------------------------------------------------------------
wxMenuItem* MainFrame::CreateImageMenuItem( wxMenu *parentMenu, wxString dataFilename, const int &id, const wxString &value) const
{
    wxMenuItem *item = new wxMenuItem(parentMenu, id, value);
    wxString ImgFileName = findFile(_T("data/menu/") + dataFilename);
    if(wxFileExists(ImgFileName))
    {
        wxImage img(ImgFileName, wxBITMAP_TYPE_PNG);
        item->SetBitmap(wxBitmap(img));
    }

    return item;
}
//------------------------------------------------------------------------------
MainFrame::~MainFrame()
{
    delete m_DevisManager;
}
//------------------------------------------------------------------------------
void MainFrame::OnUndo(wxCommandEvent& event)
{
    if( m_DevisManager->BackupCanGoTo(-1))
    {
        m_DevisManager->BackupLoad(-1);
    }

    MyRefresh();
}
//------------------------------------------------------------------------------
void MainFrame::OnRedo(wxCommandEvent& event)
{
    if( m_DevisManager->BackupCanGoTo(1))
    {
        m_DevisManager->BackupLoad(1);
    }

    MyRefresh();
}
//------------------------------------------------------------------------------
#if defined USE_MULTIPLE_CONFIG
void MainFrame::OnDialogMultiCfgManager(wxCommandEvent& event)
{
    try
    {
        DialogMultiCfgManager MyDialogMultiCfgManager(this, wxID_ANY, m_DevisManager->GetGeneralCfg());
        MyDialogMultiCfgManager.ShowModal();

        if(m_DevisManager->GetGeneralCfg()->IsModified())
        {
            wxMessageBox(_("Modified"));

            // On sauvegarde tous de suite, si modifié.
            if(findConfigFile(_T("generalCfg.xml"), false) == _T("")) // Recherche autre que data dir
            {
                wxString newFilename = copyToUserConfigDir(findFile(_T("data/generalCfg.xml")));
                m_DevisManager->GetGeneralCfg()->SetFilename(filenameWxToStd(newFilename));
            }

            m_DevisManager->GetGeneralCfg()->SaveToFile();

            MyRefresh(); // Refresh !!
        }
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnMenuItemsSelectCfg(wxCommandEvent& event)
{
    for(int i = 0; i < m_MenuItemsCfgs.size(); i++)
    {
        if(m_MenuItemsCfgs[i]->GetId() == event.GetId())
        {
            m_DevisManager->GetGeneralCfg()->SelectCfg(i);
            MyRefresh();
            break;
        }
    }
}
#endif
//------------------------------------------------------------------------------
wxString MainFrame::GetBackupDescription(const int &pos)
{
    wxString description;
    switch( m_DevisManager->BackupGetType(pos))
    {
        case BACKUP_SECTION_MODIFIED:
            description = wxString::Format(_("Section %s modified"), StrStdToWx(m_DevisManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_ADDED:
            description = wxString::Format(_("Section %s added"), StrStdToWx(m_DevisManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_REMOVED:
            description = wxString::Format(_("Section %s removed"), StrStdToWx(m_DevisManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_RENAMED:
            description = wxString::Format(_("Section %s renamed"), StrStdToWx(m_DevisManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_SECTION_MOVED:
            description = wxString::Format(_("Section %s moved"), StrStdToWx(m_DevisManager->BackupGetInfoStr(pos)).c_str());
            break;
        case BACKUP_NEW:
            description = _("Start");
            break;
    }

    return wxString(_T(" (")) + description + wxString(_T(")"));
}
//------------------------------------------------------------------------------
void MainFrame::RefreshUndoRedo()
{
    /*#if defined(__WXMSW__) // Bug avec msw : les icones des menus ne sont pas gardé en cas de changement de label
    wxBitmap undoBmp = m_ItemUndo->GetBitmap();
    wxBitmap redoBmp = m_ItemRedo->GetBitmap();
    #endif*/

    if(m_DevisManager->IsOpen() && m_DevisManager->BackupCanGoTo(-1))
    {
        m_ItemUndo->SetItemLabel(m_undoStr + GetBackupDescription(-1) + m_undoKey);
    }
    else
    {
        m_ItemUndo->SetItemLabel(m_undoStr + m_undoKey);
    }

    if(m_DevisManager->IsOpen() && m_DevisManager->BackupCanGoTo(1))
    {
        m_ItemRedo->SetItemLabel(m_redoStr + GetBackupDescription(1) + m_redoKey);
    }
    else
    {
        m_ItemRedo->SetItemLabel(m_redoStr + m_redoKey);
    }

    /*#if defined(__WXMSW__) // Bug avec msw : les icones des menus ne sont pas gardé en cas de changement de label
    m_ItemUndo->SetBitmap(undoBmp);
    m_ItemRedo->SetBitmap(redoBmp);
    #endif*/

    m_ItemRedo->Enable(m_DevisManager->IsOpen() && m_DevisManager->BackupCanGoTo(1));
    m_ItemUndo->Enable(m_DevisManager->IsOpen() && m_DevisManager->BackupCanGoTo(-1));
}

//------------------------------------------------------------------------------
void MainFrame::OnLunchDBEdit(wxCommandEvent& event)
{
    #if defined(__WXMSW__)
    wxExecute( wxStandardPaths().GetDataDir() + wxString( _T("/devisDBedit.exe")), wxEXEC_SYNC);
    #elif defined(__UNIX__)
    wxString path1 = _T("../prog-devisDBedit/devisDBedit");
    wxString path2 = _T("./prog-devisDBedit/devisDBedit");

    if(wxFileExists(path1)) // Si c'est dans le dossier source
    {
        wxExecute(path1, wxEXEC_SYNC);
    }
    else if(wxFileExists(path2)) // Si c'est dans le dossier source
    {
        wxExecute(path2, wxEXEC_SYNC);
    }
    else // Si non ==> exec
    {
        wxExecute(_T("devisDBedit"), wxEXEC_SYNC);
    }
    #endif

    ReloadSectionItemDB();
}
//------------------------------------------------------------------------------
void MainFrame::ShowDevisCleanedMsg()
{
    wxMessageBox(_("Some of the items present in this Invoice / Estimate have not been loaded because their references are not present in the items data base."), _("Refresh"), wxICON_WARNING, this);
}
//------------------------------------------------------------------------------
void MainFrame::ReloadSectionItemDB()
{
    try
    {
            // Trouver le chemin des .xml pr le devis
        std::string sectionsDbPath = filenameWxToStd(findConfigFile(_T("sectionDB.xml")));
        std::string itemsDbPath = filenameWxToStd(findConfigFile(_T("itemDB.xml")));

            // Chargement
        m_DevisManager->ReloadSectionItemDB(sectionsDbPath, itemsDbPath);

            // Refresh du GUI & Clean
        if( m_DevisManager->IsOpen() )
        {
            #if defined USE_MULTIPLE_FILE
            int backSelection = m_book->GetSelection();

            for(int i = 0; i < m_book->GetPageCount(); i++)
            {
                // /!\ Il ne faut pas faire de m_book->SetSelection() avant d'avoir netoyer le devis ! (si non, des item innexistant sont appelés).
                m_MainPanel = static_cast<MainPanel*>(m_book->GetPage(i));
                m_DevisManager->SetCurrentDevisId(m_MainPanel->GetDevisId());

                bool mustShowDevisCleanedMsg = m_DevisManager->CleanDevis();

                m_book->SetSelection(i); // Selection pour montrer à l'utilisateur où on en est.

                if(mustShowDevisCleanedMsg)
                {
                    wxMessageBox(_("Some of the items present in this Invoice / Estimate have not been loaded because their references are not present in the items data base."), _("Refresh"), wxICON_WARNING, this);
                    MyRefresh(); // Refresh de la frame et du devis.
                }
            }

            m_book->SetSelection(backSelection);
            #else
            if( m_DevisManager->CleanDevis() )
            {
                ShowDevisCleanedMsg();
            }

            MyRefresh();
            #endif
        }
        else
        {
            MyRefresh(false);
        }
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        m_DevisManager->Close();
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
        MyRefresh();
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnRefresh(wxCommandEvent& event)
{
    ReloadSectionItemDB();
}
//------------------------------------------------------------------------------
void MainFrame::OnPreferences(wxCommandEvent& event)
{
    try
    {
        GeneralCfg *generalCfg = m_DevisManager->GetGeneralCfg();

        DialogInfoGeneral MyDialogInfoGeneral(this, wxID_ANY, generalCfg);
        if(MyDialogInfoGeneral.ShowModal() == wxID_OK)
        {
            // Sauvegarde de la fenêtre seulement si demandé
            MyDialogInfoGeneral.SaveCfg();
        }

        // On sauvegarde tous de suite les deux membres de la struct, si modifié.
        if(generalCfg->IsModified())
        {
            if(findConfigFile(_T("generalCfg.xml"), false) == _T("")) // Recherche autre que data dir
            {
                wxString newFilename = copyToUserConfigDir(findFile(_T("data/generalCfg.xml")));
                generalCfg->SetFilename(filenameWxToStd(newFilename));
            }
            generalCfg->SaveToFile();
        }

        MyRefresh();

    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnQuit(wxCommandEvent& event)
{
    Close();
}
//------------------------------------------------------------------------------
void MainFrame::OnNew(wxCommandEvent& event)
{
    try
    {
        #if defined USE_MULTIPLE_FILE
        m_DevisManager->New();
        m_MainPanel = new MainPanel(m_book, this, m_DevisManager, m_DevisManager->GetLastCreatedDevisId());
        m_book->AddPage(m_MainPanel, _T(""), true);
        #else
        if(!AskToSave())
        {
            return; // Rien
        }

        m_DevisManager->New();
        #endif

        if( m_DevisManager->GetGeneralCfg()->GetGeneralCfg().GetBool("startWithASection") && m_DevisManager->GetGeneralCfg()->GetGeneralCfg().GetString("startSectionName") != "")
        {
            m_DevisManager->NewSection(m_DevisManager->GetGeneralCfg()->GetGeneralCfg().GetString("startSectionName"));
        }
        MyRefresh();
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        m_DevisManager->Close();
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
        MyRefresh();
    }

}
//------------------------------------------------------------------------------
void MainFrame::OnAbout(wxCommandEvent& event)
{
    DialogAbout MyDialogAbout(this, wxID_ANY);
    MyDialogAbout.ShowModal();
}

//------------------------------------------------------------------------------
void MainFrame::OnOpen(wxCommandEvent& event)
{
    #if defined USE_MULTIPLE_FILE
    wxFileDialog myFileDialog(this, _("Select your file"), StrStdToWx(m_DevisManager->GetGeneralCfg()->GetGeneralCfg().GetString("defaultSaveDir")), _T(""), _("Devis Maker files (*.devismaker)|*.devismaker|All files|*"), wxFD_OPEN | wxFD_MULTIPLE );
    if(myFileDialog.ShowModal() == wxID_OK)
    {
        wxArrayString filenames;
        myFileDialog.GetPaths(filenames);

        for(int i = 0; i < filenames.GetCount(); i++)
        {
            LoadFile(filenames[i]);
        }
    }
    #else
    if(!AskToSave())
    {
        return; // Rien
    }

    wxString selectedFile = wxFileSelector(_("Select your file"), StrStdToWx(m_DevisManager->GetGeneralCfg()->GetGeneralCfg().GetString("defaultSaveDir")), _T(""), _("Invoice & Estimate Maker files (*.devismaker)|*.devismaker"), _("Devis Maker files (*.devismaker)|*.devismaker|All files|*"), wxFD_OPEN, this);
    if(selectedFile != _T(""))
    {
        LoadFile(selectedFile);
    }
    #endif
}
//------------------------------------------------------------------------------

void MainFrame::OnSave(wxCommandEvent& event)
{
    if(!wxFileExists(filenameStdToWx(m_DevisManager->GetFilename())))
    {
        OnSaveAs(event);
        return;
    }

    try
    {
        m_DevisManager->Save();
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error while loading file : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }

    MyRefresh(false);
}
//------------------------------------------------------------------------------

void MainFrame::OnSaveAs(wxCommandEvent& event)
{
    wxString selectedFile = wxFileSelector(_("Save..."), filenameStdToWx(m_DevisManager->GetGeneralCfg()->GetGeneralCfg().GetString("defaultSaveDir")), filenameStdToWx(m_DevisManager->GetFilename()), _("Invoice & Estimate Maker files (*.devismaker)|*.devismaker"), _("Devis Maker files (*.devismaker)|*.devismaker|All files|*"), wxFD_SAVE, this);
    if(selectedFile != _T(""))
    {
        //Tjr extension
        if(selectedFile.Mid(selectedFile.Len() - 11, selectedFile.Len()) != _T(".devismaker"))
        {
            selectedFile << _T(".devismaker");
        }

        if(wxFileExists(selectedFile) && wxMessageBox(_("The file already exists. Do you want to remplace it ?"), _("Remplace the file ?"), wxYES_NO | wxCANCEL | wxICON_QUESTION | wxYES_DEFAULT, this) != wxYES)
        {
            return;
        }

        try
        {
            m_DevisManager->Save(filenameWxToStd(selectedFile));
            MyRefresh(false);
        }
        catch(const std::exception& e) //Rattrape toutes les exceptions
        {
            wxMessageBox(_("Error while saving file : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
        }
    }
}
//------------------------------------------------------------------------------

void MainFrame::OnDevisProperties(wxCommandEvent& event)
{
    try
    {
        DialogInfoDevis MyDialogInfoDevis(this, wxID_ANY, m_DevisManager->GetDevisCfg(), &(m_DevisManager->GetGeneralCfg()->GetDefaultDevisCfg()));
        if(MyDialogInfoDevis.ShowModal() == wxID_OK)
        {
            MyDialogInfoDevis.SaveCfg();
            MyRefresh();
        }
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }

}
//------------------------------------------------------------------------------

void MainFrame::OnDevisPrices(wxCommandEvent& event)
{
    try
    {
        DialogDevisPrices MyDialogDevisPrices(this, wxID_ANY, m_DevisManager);
        MyDialogDevisPrices.ShowModal();
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
bool MainFrame::AskToSave()
{
    if(m_DevisManager->IsOpen() && m_DevisManager->IsModified())
    {
        long rep = wxMessageBox(_("Do you want to save your work ?"), _("Close"), wxICON_WARNING | wxYES_NO | wxCANCEL | wxYES_DEFAULT, this);
        if(rep == wxCANCEL)
        {
            return false; // Rien
        }
        else if(rep == wxYES)
        {
            wxCommandEvent tmpEvt;
            OnSave(tmpEvt);
        }
    }
    return true; // On peut continuer
}

//------------------------------------------------------------------------------
void MainFrame::OnClose(wxCommandEvent& event)
{
    if(!AskToSave())
    {
        return; // Rien
    }

    m_DevisManager->Close();

    #if defined USE_MULTIPLE_FILE
    m_book->DeletePage(m_book->GetPageIndex(m_MainPanel));
    #endif

    MyRefresh(false);
}
//------------------------------------------------------------------------------

void MainFrame::OnHelp(wxCommandEvent& event)
{
    wxString FileName = findFile(_T("help/fr/devismaker.hhp"));
    if(wxFileExists(FileName))
    {
        wxHtmlHelpController *myHelp = new wxHtmlHelpController(wxHF_DEFAULT_STYLE, this);
        myHelp->AddBook(FileName);
        myHelp->Display(_T("index.html"));
    }
}
//------------------------------------------------------------------------------

void MainFrame::OnFrameClose(wxCloseEvent& event)
{
    if(m_DevisManager->IsOpen()) // SI il faut fermer quelque chose
    {
        wxCommandEvent tmpEvent;

        #if defined USE_MULTIPLE_FILE
        OnCloseAll(tmpEvent);
        #else
        OnClose(tmpEvent);
        #endif
    }

    if(m_DevisManager->IsOpen()) // Si tout n'est pas fermé... annulation
    {
        event.Veto();
    }
    else
    {
        event.Skip();
    }
}

//------------------------------------------------------------------------------
void MainFrame::LoadFile(wxString selectedFile)
{
    #if defined USE_MULTIPLE_FILE // On verifie si le fichier n'est pas déjà ouvert...
    if(m_DevisManager->IsOpen())
    {
        int backPos = m_DevisManager->GetCurrentDevisId();
        bool found = false;
        std::string stdFilename = filenameWxToStd(selectedFile);

        for(int i = 0; i < m_book->GetPageCount(); i++)
        {
            m_DevisManager->SetCurrentDevisId(static_cast<MainPanel*>(m_book->GetPage(i))->GetDevisId());
            if(m_DevisManager->GetFilename() == stdFilename)
            {
                m_book->SetSelection(i);
                found = true;
                break;
            }
        }

        if(!found) // Si le fichier n'ext pas encore chargé
        {
            m_DevisManager->SetCurrentDevisId(backPos);
        }
        else
        {
            return;
        }
    }
    #endif

    if(wxFileExists(selectedFile))
    {
        try
        {
            m_DevisManager->LoadFromFile(filenameWxToStd(selectedFile));
            #if defined USE_MULTIPLE_FILE
            m_MainPanel = new MainPanel(m_book, this, m_DevisManager, m_DevisManager->GetLastCreatedDevisId());
            m_book->AddPage(m_MainPanel, _T(""), true);
            #endif

            // Si certains items n'existe,t plus ==> Devis déjà modifié.
            if(m_DevisManager->IsModified())
            {
                wxMessageBox(_("Some of the items present in this Invoice / Estimate have not been loaded because their references are not present in the items data base."), _("Open"), wxICON_WARNING, this);
            }

            MyRefresh();
        }
        catch(const std::exception& e) //Rattrape toutes les exceptions
        {
            wxMessageBox(_("Error while loading file : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
        }
    }
    else
    {
        wxMessageBox(_("File doesn't exists."), _("Open"), wxOK | wxICON_ERROR, this);
    }
}

//------------------------------------------------------------------------------
void MainFrame::OnPrintPreview(wxCommandEvent& event)
{
    #if defined USE_SECTION_ITEM_LIMIT
    if(m_DevisManager->GetSectionCount() > DEMO_MAX_SECTIONS || m_DevisManager->GetItemCountInAllSections() > DEMO_MAX_ITEMS)
    {
        wxMessageBox(wxString::Format(_("You can't use more than %d sections and %d items !"), DEMO_MAX_SECTIONS, DEMO_MAX_ITEMS), _("Demo"), wxOK, this);
        return;
    }
    #endif

    try
    {
        wxPrintPreview *myPrintPreview = new wxPrintPreview(new DevisPrintout(PROG_FULL_NAME_GETTEXT, m_DevisManager), new DevisPrintout(_("Devis Maker"), m_DevisManager), &m_printData);
        if(!myPrintPreview->Ok())
        {
            delete myPrintPreview;
            wxMessageBox(_("There was a problem previewing.\nPerhaps your current printer is not set correctly?"), _("Previewing"), wxOK, this);
            return;
        }

        wxPreviewFrame *myPreviewFrame = new wxPreviewFrame(myPrintPreview, this, _("Preview"));
        myPreviewFrame->Maximize(true);
        myPreviewFrame->Initialize();
        myPreviewFrame->Show();
        event.Skip();
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error while printing : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnPageSetup(wxCommandEvent& event)
{
    wxPageSetupDialog pageSetupDialog(this, &m_pageSetupDialogData);
    pageSetupDialog.ShowModal();
    m_pageSetupDialogData = pageSetupDialog.GetPageSetupDialogData();
}

//------------------------------------------------------------------------------
void MainFrame::OnPrint(wxCommandEvent& event)
{
    #if defined USE_SECTION_ITEM_LIMIT
    if(m_DevisManager->GetSectionCount() > DEMO_MAX_SECTIONS || m_DevisManager->GetItemCountInAllSections() > DEMO_MAX_ITEMS)
    {
        wxMessageBox(wxString::Format(_("You can't use more than %d sections and %d items !"), DEMO_MAX_SECTIONS, DEMO_MAX_ITEMS), _("Demo"), wxOK, this);
        return;
    }
    #endif

    try
    {
        wxPrinter printer(&m_printDialogData);
        DevisPrintout printout(PROG_FULL_NAME_GETTEXT, m_DevisManager);
        if (!printer.Print(this, &printout, true /*prompt*/))
        {
            if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
            {
                wxMessageBox(_("There was a problem printing.\nPerhaps your current printer is not set correctly?"), _("Printing"), wxOK, this);
            }
        }
        else
        {
            m_printDialogData = printer.GetPrintDialogData();
        }
        event.Skip();
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        wxMessageBox(_("Error while printing : ") + wxString(_T("\n")) + StrStdToWx(e.what()), _("Error..."),  wxOK | wxICON_ERROR, this);
    }
}
#if defined USE_MULTIPLE_FILE
//------------------------------------------------------------------------------
void MainFrame::RefreshNotebook()
{
    if(m_DevisManager->IsOpen())
    {
        int backPos = m_DevisManager->GetCurrentDevisId();

        for(int i = 0; i < m_book->GetPageCount(); i++)
        {
            MainPanel *currentPanel = static_cast<MainPanel*>(m_book->GetPage(i));
            m_DevisManager->SetCurrentDevisId(currentPanel->GetDevisId());

            if(m_DevisManager->GetDevisCfg()->GetString("name").empty())
            {
                m_book->SetPageText(i, _("Unnamed"));
            }
            else
            {
                m_book->SetPageText(i, StrStdToWx(m_DevisManager->GetDevisCfg()->GetString("name")) + _T(" - ") + StrStdToWx(m_DevisManager->GetDevisCfg()->GetString("numDevis")));
            }

            if(m_DevisManager->IsModified())
            {
                m_book->SetPageBitmap(i, m_bitmapToSave);
            }
            else
            {
                m_book->SetPageBitmap(i, m_bitmapSaved);
            }
        }

        m_DevisManager->SetCurrentDevisId(backPos);
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnNotebookChanged(wxAuiNotebookEvent& event)
{
    m_MainPanel = static_cast<MainPanel*>(m_book->GetPage(event.GetSelection()));
    m_DevisManager->SetCurrentDevisId(m_MainPanel->GetDevisId());
    MyRefresh(false);
}
//------------------------------------------------------------------------------
void MainFrame::OnNotebookClosing(wxAuiNotebookEvent& event)
{
    if(!AskToSave())
    {
        event.Veto(); // Rien
    }

    m_DevisManager->SetCurrentDevisId(static_cast<MainPanel*>(m_book->GetPage(event.GetSelection()))->GetDevisId());
    m_DevisManager->Close();
}
//------------------------------------------------------------------------------
void MainFrame::OnNotebookClosed(wxAuiNotebookEvent& event)
{
    /*if(m_DevisManager->IsOpen())
    {
        if(event.GetSelection() != 0)
        {
            m_book->SetSelection(event.GetSelection() - 1);
        }
        else
        {
            m_book->SetSelection(0);
        }
    }*/

    MyRefresh(false);
}
//------------------------------------------------------------------------------
void MainFrame::OnSaveAll(wxCommandEvent& event)
{
    if(m_DevisManager->IsOpen())
    {
        int backSelection = m_book->GetSelection();

        for(int i = 0; i < m_book->GetPageCount(); i++)
        {
            m_book->SetSelection(i);
            OnSave(event);
        }

        m_book->SetSelection(backSelection);
    }
}
//------------------------------------------------------------------------------
void MainFrame::OnCloseAll(wxCommandEvent& event)
{
    if(m_DevisManager->IsOpen())
    {
        for(int i = m_book->GetPageCount() - 1; i >= 0; i--)
        {
            m_book->SetSelection(i);
            OnClose(event);
        }
    }
}
//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------
//----------------------- My drop target ---------------------------------------
//------------------------------------------------------------------------------
MyFileDropTarget::MyFileDropTarget(MainFrame *frame)
{
    m_frame = frame;
}
//------------------------------------------------------------------------------
bool MyFileDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
{
    #if defined USE_MULTIPLE_FILE
    for(int i = 0; i < filenames.Count(); i++)
    {
        m_frame->LoadFile(filenames[0]);
    }
    #else
    if(filenames.Count() == 1)
    {
        if(!m_frame->AskToSave())
        {
            return false; // Rien
        }

        m_frame->LoadFile(filenames[0]);
    }
    else
    {
        wxMessageBox(_("You can't load more than one file at once !"), _("Open"), wxOK | wxICON_ERROR, m_frame);
        return false;
    }
    #endif

    return true;
}
//------------------------------------------------------------------------------

