///File : Image.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Image.hpp"
//------------------------------------------------------------------------------
namespace mdo
{

//------------------------------------------------------------------------------
//---------------------------------- Image -------------------------------------
//------------------------------------------------------------------------------
bool Image::CalculSizeReal(wxDC *dc, const int &width, const int &askedHeight)
{
    if(wxFileExists(m_data.filename) && m_data.img.LoadFile(m_data.filename, wxBITMAP_TYPE_ANY))
    {
        if(m_data.isStretch) // On étire l'image pour qu'elle rentre dans le cadre.
        {
            double askedRatio = double(width) / double(askedHeight);

            m_data.imgResizeRatioX = double(width) / double(m_data.img.GetWidth());
            m_data.imgResizeRatioY = double(askedHeight) / double(m_data.img.GetHeight());
        }
        else // On étire pas l'image
        {
            // Est-ce la hauteur ou la largeur qui "coinse" ?
            double askedRatio = double(width) / double(askedHeight);
            double imageRatio = double(m_data.img.GetWidth()) / double(m_data.img.GetHeight());

            if(askedRatio < imageRatio) // On modifie en fontion de la width
            {
                m_data.imgResizeRatioX = m_data.imgResizeRatioY = double(width) / double(m_data.img.GetWidth());
            }
            else // On modifie en fontion de la height
            {
                m_data.imgResizeRatioX = m_data.imgResizeRatioY = double(askedHeight) / double(m_data.img.GetHeight());
            }
        }
    }

    m_sizeReal.SetWidth(width);
    m_sizeReal.SetHeight(askedHeight);

    m_isReadyToDraw = true;
}
//------------------------------------------------------------------------------
Image::Image(ConfigManager *cfg, ConfigManager *devisCfg, DrawableObject *parrentSizer, const std::string &name) : DrawableObject(cfg, devisCfg, parrentSizer, name)
{
    m_typeName = MDO_IMAGE_TYPE_NAME;
    SetConfigName(""); // Rechargement du prefix
}
//------------------------------------------------------------------------------
void Image::LoadFromCfg(const std::string &name)
{
    DrawableObject::LoadFromCfg(name);
    m_data.isBorder = m_cfg->GetBool(m_cfgPrefix + "isBorder");
    m_data.isStretch = m_cfg->GetBool(m_cfgPrefix + "isStretch");
    m_data.filename = StrStdToWx(m_cfg->GetString(m_cfgPrefix + "filename"));

    m_data.vAlign = m_cfg->GetInt(m_cfgPrefix + "vAlign");
    m_data.hAlign = m_cfg->GetInt(m_cfgPrefix + "hAlign");
}
//------------------------------------------------------------------------------
void Image::SaveToCfg(const std::string &name)
{
    DrawableObject::SaveToCfg(name);
    m_cfg->SetBool(m_cfgPrefix + "isBorder", m_data.isBorder);
    m_cfg->SetBool(m_cfgPrefix + "isStretch", m_data.isStretch);
    m_cfg->SetString(m_cfgPrefix + "filename", StrWxToStd(m_data.filename));

    m_cfg->SetInt(m_cfgPrefix + "vAlign", m_data.vAlign);
    m_cfg->SetInt(m_cfgPrefix + "hAlign", m_data.hAlign);
}
//------------------------------------------------------------------------------
void Image::CreateToCfg(const std::string &name)
{
    DrawableObject::CreateToCfg(name);
    m_cfg->SetBool(m_cfgPrefix + "isBorder", false);
    m_cfg->SetBool(m_cfgPrefix + "isStretch", false);
    m_cfg->SetString(m_cfgPrefix + "filename", StrWxToStd(findFile(_T("data/example-img.png"))));

    m_cfg->SetInt(m_cfgPrefix + "vAlign", vAlign_Left);
    m_cfg->SetInt(m_cfgPrefix + "hAlign", hAlign_Top);

    LoadFromCfg();
}
//------------------------------------------------------------------------------
void Image::RemoveFromCfg(const std::string &name, const bool &withChilds)
{
    DrawableObject::RemoveFromCfg(name);
    m_cfg->RemoveBool(m_cfgPrefix + "isBorder");
    m_cfg->RemoveBool(m_cfgPrefix + "isStretch");
    m_cfg->RemoveString(m_cfgPrefix + "filename");

    m_cfg->RemoveInt(m_cfgPrefix + "vAlign");
    m_cfg->RemoveInt(m_cfgPrefix + "hAlign");
}
//------------------------------------------------------------------------------
void Image::EditPropertyDialog(DialogProperty *window)
{
    DrawableObject::EditPropertyDialog(window);

    wxPanel *PagePanel;
    wxBoxSizer *PageSizer;

    // Special Mdo
    PagePanel = new wxPanel(window->GetNotebook(), wxID_ANY);
    PageSizer = new wxBoxSizer(wxVERTICAL);

    window->AddParamCtrl(type_check, PagePanel, PageSizer, m_cfgPrefix + "isBorder", _("Add border"));
    window->AddParamCtrl(type_check, PagePanel, PageSizer, m_cfgPrefix + "isStretch", _("Stretch the image"));
    window->AddParamCtrl(type_file, PagePanel, PageSizer, m_cfgPrefix + "filename", _("Image filename :"));

    PageSizer->AddSpacer(20);

    wxArrayString wxVAlignNames(vAlign_nb, vAlign_Names);
    window->AddParamCtrl(type_choice_index, PagePanel, PageSizer, m_cfgPrefix + "vAlign", _("Vertical align :"), static_cast<void*>(&wxVAlignNames));

    wxArrayString wxHAlignNames(hAlign_nb, hAlign_Names);
    window->AddParamCtrl(type_choice_index, PagePanel, PageSizer, m_cfgPrefix + "hAlign", _("Horizontal align :"), static_cast<void*>(&wxHAlignNames));

    PagePanel->SetSizer(PageSizer);
    window->GetNotebook()->AddPage(PagePanel, _("Object"));
}
//------------------------------------------------------------------------------
void Image::Draw(wxDC *dc, const wxPoint &pos)
{
    /*ThrowIfNotReadyToDraw();
    wxImage newImg = m_data.img.Rescale(m_data.img.GetWidth() * m_data.imgResizeRatio, m_data.img.GetHeight() * m_data.imgResizeRatio);


    // Pos
    wxPoint drawPos;
    switch(m_data.vAlign)
    {
        case vAlign_Left:
            drawPos.x = pos.x;
            break;
        case align_Center:
            drawPos.x = pos.x + (GetSizeReal().GetWidth() - newImg.GetWidth()) / 2;
            break;
        case vAlign_Right:
            drawPos.x = pos.x + GetSizeReal().GetWidth() - newImg.GetWidth();
            break;
        default:
            std::string msg = "Unknown valign " + nbToString(m_data.vAlign);
            throw ticpp::Exception(msg.c_str());
            break;
    }

    switch(m_data.hAlign)
    {
        case hAlign_Top:
            drawPos.y = pos.y;
            break;
        case align_Center:
            drawPos.y = pos.y + (GetSizeReal().GetHeight() - newImg.GetHeight()) / 2;
            break;
        case hAlign_Bottom:
            drawPos.y = pos.y + GetSizeReal().GetHeight() - newImg.GetHeight();
            break;
        default:
            std::string msg = "Unknown halign " + nbToString(m_data.hAlign);
            throw ticpp::Exception(msg.c_str());
            break;
    }

    // Draw
    dc->DrawBitmap(wxBitmap(newImg), drawPos.x, drawPos.y, true);

    if(m_data.isBorder)
    {
        // Bordures
        dc->DrawRoundedRectangle(pos.x, pos.y, GetSizeReal().GetWidth(), GetSizeReal().GetHeight(), m_cfg->GetInt("arcSize"));
    }*/

    ThrowIfNotReadyToDraw();

    double backScaleX;
    double backScaleY;
    dc->GetUserScale(&backScaleX, &backScaleY);

    double newScaleX = m_data.imgResizeRatioX * backScaleX;
    double newScaleY = m_data.imgResizeRatioY * backScaleY;

    dc->SetUserScale(newScaleX, newScaleY);

    // Pos
    wxPoint drawPos;
    double xPosResize = (1.f / newScaleX) * backScaleX;
    switch(m_data.vAlign)
    {
        case vAlign_Left:
            drawPos.x = pos.x * xPosResize;
            break;
        case vAlign_Center:
            drawPos.x = pos.x * xPosResize + (GetSizeReal().GetWidth() * xPosResize - m_data.img.GetWidth()) / 2;
            break;
        case vAlign_Right:
            drawPos.x = pos.x * xPosResize + GetSizeReal().GetWidth() * xPosResize - m_data.img.GetWidth();
            break;
        default:
            std::string msg = "Unknown valign " + nbToString(m_data.vAlign);
            throw ticpp::Exception(msg.c_str());
            break;
    }

    double yPosResize = (1.f / newScaleY) * backScaleY;
    switch(m_data.hAlign)
    {
        case hAlign_Top:
            drawPos.y = pos.y * yPosResize;
            break;
        case hAlign_Center:
            drawPos.y = pos.y * yPosResize + (GetSizeReal().GetHeight() * yPosResize - m_data.img.GetHeight()) / 2;
            break;
        case hAlign_Bottom:
            drawPos.y = pos.y * yPosResize + GetSizeReal().GetHeight() * yPosResize - m_data.img.GetHeight();
            break;
        default:
            std::string msg = "Unknown halign " + nbToString(m_data.hAlign);
            throw ticpp::Exception(msg.c_str());
            break;
    }

    // Draw
    dc->DrawBitmap(wxBitmap(m_data.img), drawPos.x, drawPos.y, true);

    // On revien à la normal
    dc->SetUserScale(backScaleX, backScaleY);

    if(m_data.isBorder)
    {
        // Bordures
        dc->DrawRoundedRectangle(pos.x, pos.y, GetSizeReal().GetWidth(), GetSizeReal().GetHeight(), m_cfg->GetInt("arcSize"));
    }
}
//------------------------------------------------------------------------------


} // namespace mdo
